/*
 * Copyright 2006-2007, Thomas Maier-Komor
 * LICENSE: GPLv2, see file COPYING for details
 */

#include <assert.h>
#include <errno.h>
#include <fcntl.h>
#include <libgen.h>
#include <poll.h>
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <unistd.h>

#include "log.h"
#include "network.h"
#include "sysstat.h"
#include "terminal.h"


int Localmode = 0;
unsigned Interval = 5;

static sysstat_t
	*SS = 0, *Current = 0;


static void
usage(int r)
{
	printf(
	"sysstatd [option] [interval [count]]\n"
	"Options:\n"
	"-h        : display this help screen and exit\n"
	"-k        : kill sysstat's daemon\n"
	"-p        : operate in pipe mode\n"
	"-v        : operate in verbose mode\n"
	"-V        : display version number and exit\n"
	);
	exit(r);
}


static void
version(void)
{
	printf(
	"sysstat version "VERSION"\n"
	"Copyright 2006-2007, Thomas Maier-Komor\n"
	"This software is licensed under the GPLv2. See file COPYING for details.\n"
	);
	exit(EXIT_SUCCESS);
}


int
main(int argc, char **argv)
{
	unsigned daemon = 1, multicast = 0, iter = 0, killdaemon = 0;
	int c, count = 0, pid, sock;
	extern char *optarg;
	FILE *pidf;

	if (-1 == chdir("/var/tmp"))
		error("Could not change directory to /var/tmp: %s\n",strerror(errno));
	while ((c = getopt(argc, argv, "dhklmpsvV")) != -1) {
		extern int Port;
		extern char *McastAddr;

		switch(c) {
		default:
			assert(0);
			break;
		case 'd':
			daemon = 0;
		case 'h':
			usage(EXIT_SUCCESS);
			break;
		case 'k':
			dbug("kill daemon requested\n");
			killdaemon = 1;
			break;
		case 'l':
			Localmode = 1;
			dbug("local mode requested\n");
			break;
		case 'm':
			McastAddr = optarg;
			info("multicast address set to %s\n",McastAddr);
			break;
		case 'p':
			if (1 != sscanf(optarg,"%u",&Port))
				usage(EXIT_FAILURE);
			break;
		case 's':
			dbug("logging via syslog\n");
			log2syslog(basename(argv[0]));
			break;
		case 'v':
			setVerbose("sysstatd.log");
			break;
		case 'V':
			version();
			break;
		case '?':
			usage(EXIT_FAILURE);
			break;
		}
	}
	if (optind < argc) {
		if (1 != sscanf(argv[optind],"%u",&Interval))
			usage(EXIT_FAILURE);
		dbug("interval set to %u\n",Interval);
		++optind;
	}
	if (optind < argc) {
		if (1 != sscanf(argv[optind],"%d",&count) || (count <= 0))
			usage(EXIT_FAILURE);
		dbug("count set to %d\n",count);
		++optind;
	}
	if (optind < argc) {
		usage(EXIT_FAILURE);
	}
	if (Localmode)
		rundaemon(Interval,count);
	pidf = fopen("/tmp/sysstat.pid","r");
	if (pidf) {
		char pn[] = "/proc/\0\0\0\0\0\0",*p;
		struct stat st;
		info("/tmp/sysstat.pid exists\n");
		if (1 != fscanf(pidf,"%d",&pid)) {
			warn("corrupt pid file /tmp/systat.pid - unlinking it\n");
			if (-1 == unlink("/tmp/sysstat.pid"))
				error("cannot unlink stale pid file /tmp/sysstat.pid: %s\n",strerror(errno));
		} else {
			p = pn + 6;
			snprintf(p,6,"%d",pid);
			dbug("stat(%s,...)\n",pn);
			if (-1 == stat(pn,&st)) {
				info("unlinking stale pid file /tmp/sysstat.pid\n");
				if (-1 == unlink("/tmp/sysstat.pid"))
					error("cannot unlink stale pid file /tmp/sysstat.pid: %s\n",strerror(errno));
				multicast = 1;
			} else {
				info("daemon already running with pid %d\n",pid);
				if (killdaemon) {
					if (0 == kill(pid,SIGTERM)) {
						info("sending SIGTERM to %d\n",pid);
						exit(EXIT_SUCCESS);
					} else {
						error("cannot send SIGTERM to %d: %s\n",pid,strerror(errno));
					}
				}
				if (daemon)
					exit(EXIT_FAILURE);
			}
		}
		fclose(pidf);
	}
	if (killdaemon) {
		error("cannot kill sysstat daemon: no sysstat daemon running\n");
		exit(EXIT_FAILURE);
	}
	if (daemon != 0) {
		pid_t pid, dpid;
		if ((pid = fork())) {
			int status;
			waitpid(pid, &status, 0);
			assert(status == 0);
		} else if (dpid = fork()) {
			FILE *pidf;
			pidf = fopen("/tmp/sysstat.pid","w");
			if (pidf == 0)
				error("cannot create pid file: %s\n",strerror(errno));
			fprintf(pidf,"%d\n",dpid);
			fclose(pidf);
			chmod("/tmp/sysstat.pid",0666);
			exit(EXIT_SUCCESS);
		} else {
			rundaemon(Interval,count);
		} 
		exit(EXIT_SUCCESS);
	} else
		rundaemon(Interval,count);
	return 0;
}

// vim:tw=0
